@@ -44,55 +44,38 @@
   (add-extended-slot '() down)
   (add-extended-slot '() down)
 
-  (set! HORIZPOS (get-horiz-start))
-  (set! VERTPOS (get-vert-start))
+  (set! HORIZPOS 0)
+  (set! VERTPOS 0)
 
-  (set! VERTPOS (inexact->exact (truncate (+ VERTPOS
-					     (/ (get-card-height) 2)))))
-  (set! HORIZPOS (inexact->exact (truncate (+ HORIZPOS 
-					      (* (/ 15 2) 
-						 (get-card-width))))))
+  (set! VERTPOS (+ VERTPOS 0.5))
+  (set! HORIZPOS (+ HORIZPOS 7))
   (add-normal-slot '())
   (add-carriage-return-slot)
-  (set! HORIZPOS (inexact->exact (truncate (+ HORIZPOS 
-					      (* (/ 15 2) 
-						 (get-card-width))))))
+  (set! HORIZPOS (+ HORIZPOS 7))
   (add-normal-slot '())
   (add-carriage-return-slot)
-  (set! HORIZPOS (inexact->exact (truncate (+ HORIZPOS 
-					      (* (/ 15 2) 
-						 (get-card-width))))))
+  (set! HORIZPOS (+ HORIZPOS 7))
   (add-normal-slot '())
   (add-carriage-return-slot)
-  (set! HORIZPOS (inexact->exact (truncate (+ HORIZPOS 
-					      (* (/ 15 2) 
-						 (get-card-width))))))
+  (set! HORIZPOS (+ HORIZPOS 7))
 
 
-  (set! HORIZPOS (get-horiz-start))
-  (set! VERTPOS (get-vert-start))
+  (set! HORIZPOS 0)
+  (set! VERTPOS 0)
 
-  (set! HORIZPOS (inexact->exact (truncate (+ HORIZPOS 
-					      (* (/ 15 2) 
-						 (get-card-width))))))
+  (set! HORIZPOS (+ HORIZPOS 7))
   (add-blank-slot)
   (add-normal-slot '())
   (add-carriage-return-slot)
-  (set! HORIZPOS (inexact->exact (truncate (+ HORIZPOS 
-					      (* (/ 15 2) 
-						 (get-card-width))))))
+  (set! HORIZPOS (+ HORIZPOS 7))
   (add-blank-slot)
   (add-normal-slot '())
   (add-carriage-return-slot)
-  (set! HORIZPOS (inexact->exact (truncate (+ HORIZPOS 
-					      (* (/ 15 2) 
-						 (get-card-width))))))
+  (set! HORIZPOS (+ HORIZPOS 7))
   (add-blank-slot)
   (add-normal-slot '())
   (add-carriage-return-slot)
-  (set! HORIZPOS (inexact->exact (truncate (+ HORIZPOS 
-					      (* (/ 15 2) 
-						 (get-card-width))))))
+  (set! HORIZPOS (+ HORIZPOS 7))
   (add-blank-slot)
   (add-normal-slot '())
 
@@ -117,22 +100,22 @@
 (define (get-base-string)
   (cond ((and (> BASE-VAL 1)
 	      (< BASE-VAL 11))
-	 (string-append (_"Base Card:  ") (number->string BASE-VAL)))
+	 (string-append (_"Base Card:") " " (number->string BASE-VAL)))
 	((= BASE-VAL 1)
-	 (_"Base Card:  Ace"))
+	 (_"Base Card: Ace"))
 	((= BASE-VAL 11)
-	 (_"Base Card:  Jack"))
+	 (_"Base Card: Jack"))
 	((= BASE-VAL 12)
-	 (_"Base Card:  Queen"))
+	 (_"Base Card: Queen"))
 	((= BASE-VAL 13)
-	 (_"Base Card:  King"))
+	 (_"Base Card: King"))
 	(#t #f)))
 
 (define (get-stock-no-string)
   (if (> (length (get-cards 0)) 1)
-      (string-append (_"Stock left:  ") 
+      (string-append (_"Stock left:") " " 
 		     (number->string (length (get-cards 0))))
-      (string-append (_"Stock left:  0")))) 
+      (string-append (_"Stock left: 0")))) 
 
 (define (button-pressed slot-id card-list)
   (and (not (empty-slot? slot-id))
