@@ -40,6 +40,7 @@
 #include "games-frame.h"
 #include "games-files.h"
 #include "games-gridframe.h"
+#include "games-preimage.h"
 #include "glines.h"
 
 #define KEY_DIR "/apps/glines"
@@ -61,8 +62,8 @@ static GtkWidget *app, *appbar, *pref_dialog;
 GtkWidget *preview_widgets[3];
 field_props field[FIELDSIZE * FIELDSIZE];
 
-/* The unscaled pixbuf as read from file. Cached here to save file reads. */
-GdkPixbuf *raw_pixbuf = NULL;
+/* Pre-rendering image data prepared from file. */
+GamesPreimage *ball_preimage = NULL;
 /* The tile images with balls rendered on them. */
 GdkPixmap *ball_pixmap = NULL;
 /* The balls rendered to a size appropriate for the preview. */
@@ -78,6 +79,9 @@ int active = -1;
 int target = -1;
 int inmove = 0;
 int score = 0;
+int cursor_x = FIELDSIZE / 2;
+int cursor_y = FIELDSIZE / 2;
+gboolean show_cursor = FALSE;
 
 int boxsize;
 
@@ -115,10 +119,10 @@ GnomeUIInfo mainmenu[];
 
 static void 
 load_image (gchar *fname,
-	    GdkPixbuf **pixbuf)
+	    GamesPreimage **preimage)
 {
 	gchar *tmp, *fn = NULL;
-	GdkPixbuf *image;
+	GamesPreimage *tmp_preimage;
 
 	tmp = g_build_filename ("glines", fname, NULL);
 	
@@ -126,41 +130,36 @@ load_image (gchar *fname,
 	g_free (tmp);
 
 
-	/* Just think of this as an if statement, but we want to use break. */
-	while (!g_file_test (fn, G_FILE_TEST_EXISTS)) {
+	if (!g_file_test (fn, G_FILE_TEST_EXISTS)) {
 		char * message;
 		GtkWidget * w;
-
-		/* ball.png was replaced with balls.svg. */
-		if (g_utf8_collate (fname, "ball.png") == 0) {
-			tmp = g_build_filename ("glines", "balls.svg", NULL);
-			fn = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, (tmp), FALSE, NULL);
-			g_free (tmp);
-			if (g_file_test (fn, G_FILE_TEST_EXISTS))
-				break;
+		
+		tmp = g_build_filename ("glines", "balls.svg", NULL);
+		fn = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, (tmp), FALSE, NULL);
+		g_free (tmp);
+		if (!g_file_test (fn, G_FILE_TEST_EXISTS)) {
+			message = g_strdup_printf (_("Five or More could not find image file:\n%s\n\n"
+						     "Please check Five or More is installed correctly."), fn);
+			w = gtk_message_dialog_new (GTK_WINDOW (app),
+						    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+						    GTK_MESSAGE_ERROR,
+						    GTK_BUTTONS_OK,
+						    message,
+						    NULL);
+			gtk_dialog_run (GTK_DIALOG (w));	
+			g_free (message);
+			exit (1);
 		}
-
-		message = g_strdup_printf (_("Five or More could not find image file:\n%s\n\n"
-					     "Please check Five or More is installed correctly."), fn);
-		w = gtk_message_dialog_new (GTK_WINDOW (app),
-					    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-					    GTK_MESSAGE_ERROR,
-					    GTK_BUTTONS_OK,
-					    message,
-					    NULL);
-		gtk_dialog_run (GTK_DIALOG (w));	
-		g_free (message);
-		exit (1);
 	}
 
-	image = gdk_pixbuf_new_from_file (fn, NULL);
+	tmp_preimage = games_preimage_new_from_uri (fn, NULL);
 	g_free( fn );
 
 
-	if (*pixbuf)
-		g_object_unref (*pixbuf);
+	if (*preimage)
+		g_object_unref (*preimage);
 
-	*pixbuf = image;
+	*preimage = tmp_preimage;
 }
 
 static void
@@ -175,9 +174,8 @@ refresh_pixmaps (void)
 	if (!ball_pixmap)
 		return;
 
-	ball_pixbuf = gdk_pixbuf_scale_simple (raw_pixbuf, 4*boxsize, 
-					       7*boxsize, 
-					       GDK_INTERP_BILINEAR);
+	ball_pixbuf = games_preimage_render (ball_preimage, 4*boxsize, 
+					     7*boxsize, NULL);
 
 	gc = gdk_gc_new (ball_pixmap);
 	gdk_gc_set_foreground (gc, &backgnd.color);
@@ -214,9 +212,8 @@ refresh_preview_pixmaps (void)
 		for (i=0; i<7; i++)
 			g_object_unref (preview_pixmaps[i]);
 
-	scaled = gdk_pixbuf_scale_simple (raw_pixbuf, 4*preview_width, 
-					  7*preview_height, 
-					  GDK_INTERP_BILINEAR);
+	scaled = games_preimage_render (ball_preimage, 4*preview_width, 
+				        7*preview_height, NULL);
 
 	for (i=0; i<7; i++) {
 		preview_pixmaps[i] = gdk_pixmap_new (widget->window,
@@ -330,7 +327,14 @@ draw_preview (void)
 }
 
 static void
-show_scores (gint pos)
+game_new_callback (GtkWidget *widget, void *data)
+{
+	reset_game ();
+ 	start_game ();
+}
+
+static void
+show_scores (gint pos, gboolean new_game)
 {
 	GtkWidget *dialog;
 
@@ -339,6 +343,10 @@ show_scores (gint pos)
 		gtk_window_set_transient_for (GTK_WINDOW (dialog),
 					      GTK_WINDOW (app));
 		gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+		if (new_game)
+			g_signal_connect (G_OBJECT (dialog), "destroy", 
+					  G_CALLBACK (game_new_callback), 
+					  NULL);
 	}
 }
 
@@ -365,7 +373,7 @@ game_over (void)
 
 	gnome_appbar_set_status (GNOME_APPBAR (appbar), _("Game Over!"));
 	pos = gnome_score_log (score, NULL, TRUE);
-	show_scores (pos);
+	show_scores (pos, TRUE);
 	update_score_state ();
 	return;
 }
@@ -496,23 +504,10 @@ deactivate (GtkWidget *widget, int x, int y)
 	active = -1;
 }
 
-static gint
-button_press_event (GtkWidget *widget, GdkEvent *event)
+static void cell_clicked (GtkWidget *widget, int fx, int fy)
 {
 	int x, y;
-	int fx, fy;
 
-	/* XXX Ezt megkapja */
-
-	if(inmove) return TRUE;
-
-	/* Ignore the 2BUTTON and 3BUTTON events. */
-	if (event->type != GDK_BUTTON_PRESS)
-		return TRUE;
-	
-	gtk_widget_get_pointer (widget, &x, &y);
-	fx = x / boxsize;
-	fy = y / boxsize;
         gnome_appbar_set_status(GNOME_APPBAR(appbar), "");
 	if(field[fx + fy*9].color == 0)
 	{
@@ -568,6 +563,118 @@ button_press_event (GtkWidget *widget, GdkEvent *event)
 		}
 	}
 
+}
+
+static gint
+button_press_event (GtkWidget *widget, GdkEvent *event)
+{
+	int x, y;
+	int fx, fy;
+
+	if(inmove) return TRUE;
+
+	/* Ignore the 2BUTTON and 3BUTTON events. */
+	if (event->type != GDK_BUTTON_PRESS)
+		return TRUE;
+	
+	if (show_cursor)
+	{
+		show_cursor = FALSE;
+		draw_box (draw_area, cursor_x, cursor_y);
+	}
+
+	gtk_widget_get_pointer (widget, &x, &y);
+	fx = x / boxsize;
+	fy = y / boxsize;
+
+	cursor_x = fx;
+	cursor_y = fy;
+
+	cell_clicked (widget, fx, fy);
+
+	return TRUE;
+}
+
+static void
+move_cursor (int dx, int dy)
+{
+	int old_x = cursor_x;
+	int old_y = cursor_y;
+
+	if (!show_cursor)
+	{
+		show_cursor = TRUE;
+		draw_box (draw_area, cursor_x, cursor_y);
+	}
+
+	cursor_x = cursor_x + dx;
+	if (cursor_x < 0)
+		cursor_x = 0;
+	if (cursor_x >= FIELDSIZE)
+		cursor_x = FIELDSIZE - 1;
+	cursor_y = cursor_y + dy;
+	if (cursor_y < 0)
+		cursor_y = 0;
+	if (cursor_y >= FIELDSIZE)
+		cursor_y = FIELDSIZE - 1;
+
+	if (cursor_x == old_x && cursor_y == old_y)
+		return;
+
+	draw_box (draw_area, old_x, old_y);
+	draw_box (draw_area, cursor_x, cursor_y);
+}
+
+static gboolean
+key_press_event (GtkWidget *widget, GdkEventKey *event, void *d)
+{
+	guint key;
+
+	key = event->keyval;
+
+	switch (key)
+	{
+		case GDK_Left:
+		case GDK_KP_Left:
+			move_cursor (-1, 0);
+			break;
+		case GDK_Right:
+		case GDK_KP_Right:
+			move_cursor (1, 0);
+			break;
+		case GDK_Up:
+		case GDK_KP_Up:
+			move_cursor (0, -1);
+			break;
+		case GDK_Down:
+		case GDK_KP_Down:
+			move_cursor (0, 1);
+			break;
+		case GDK_Home:
+		case GDK_KP_Home:
+			move_cursor (-999, 0);
+			break;
+		case GDK_End:
+		case GDK_KP_End:
+			move_cursor (999, 0);
+			break;
+		case GDK_Page_Up:
+		case GDK_KP_Page_Up:
+			move_cursor (0, -999);
+			break;
+		case GDK_Page_Down:
+		case GDK_KP_Page_Down:
+			move_cursor (0, 999);
+			break;
+
+		case GDK_space:
+		case GDK_Return:
+		case GDK_KP_Enter:
+			if (show_cursor)
+				cell_clicked (widget, cursor_x, cursor_y);
+			break;
+	}
+
 	return TRUE;
 }
 
@@ -647,6 +754,23 @@ field_expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer gp)
 			}
 		}
 	}
+
+	if (show_cursor && cursor_x >= x_start && cursor_x <= x_end
+		&& cursor_y >= y_start && cursor_y <= y_end)
+	{
+		GdkColormap *cmap;
+		GdkColor color;
+
+		gdk_color_parse ("#FFFFFF", &color);
+		cmap = gtk_widget_get_colormap (widget);
+		gdk_colormap_alloc_color (cmap, &color, FALSE, TRUE);
+		gdk_gc_set_foreground (gc, &color);
+
+		gdk_draw_rectangle (window, gc, FALSE,
+			cursor_x * boxsize + 1, cursor_y * boxsize + 1,
+			boxsize - 2, boxsize - 2);
+	}
+
         g_object_unref (gc);
 
 	draw_grid ();
@@ -924,17 +1048,10 @@ animate (gpointer gp)
 	return TRUE;
 }
 
-static void
-game_new_callback (GtkWidget *widget, void *data)
-{
-	reset_game ();
- 	start_game ();
-}
-
 static void
 game_top_ten_callback(GtkWidget *widget, gpointer data)
 {
-	show_scores (0);
+	show_scores (0, FALSE);
 }
 
 static int
@@ -1051,7 +1168,7 @@ bg_color_callback (GtkWidget *widget, gpointer data)
 static void
 load_theme ()
 {
-	load_image (ball_filename, &raw_pixbuf);
+	load_image (ball_filename, &ball_preimage);
 	refresh_pixmaps ();
 	refresh_preview_pixmaps ();
 }
@@ -1216,17 +1333,13 @@ preview_configure_cb (GtkWidget * widget, GdkEventConfigure * event)
 static int
 configure_event_callback (GtkWidget *widget, GdkEventConfigure *event)
 {
-	int xboxsize, yboxsize;
-
 	if (ball_pixmap) 
 		g_object_unref (ball_pixmap);
 
 	if (blank_pixmap)
 		g_object_unref (blank_pixmap);
 
-	xboxsize = event->width/FIELDSIZE;
-	yboxsize = event->height/FIELDSIZE;
-	boxsize = MIN (xboxsize, yboxsize);
+	boxsize = (event->width - 1)/FIELDSIZE;
 
 	ball_pixmap = gdk_pixmap_new (draw_area->window, boxsize*4, boxsize*7,
 				      -1);
@@ -1534,7 +1647,7 @@ main (int argc, char *argv [])
 
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_table_attach_defaults (GTK_TABLE (table), hbox, 0, 1, 0, 1);
-	label = gtk_label_new (g_strdup_printf ("<span weight=\"bold\">%s</span>", _("Next Balls:")));
+	label = gtk_label_new (g_strdup_printf ("<span weight=\"bold\">%s</span>", _("Next:")));
 	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
 
@@ -1571,15 +1684,21 @@ main (int argc, char *argv [])
 	draw_area = gtk_drawing_area_new ();
 	g_signal_connect (G_OBJECT(draw_area), "button_press_event",
 			  G_CALLBACK (button_press_event), NULL);
+	g_signal_connect (G_OBJECT(draw_area), "key_press_event",
+			  G_CALLBACK (key_press_event), NULL);
 	g_signal_connect (G_OBJECT (draw_area), "configure_event",
 			  G_CALLBACK (configure_event_callback), NULL);
 	g_signal_connect (G_OBJECT (draw_area), "expose_event",
 			  G_CALLBACK (field_expose_event), NULL);
 	gridframe = games_grid_frame_new (FIELDSIZE, FIELDSIZE);
+	games_grid_frame_set_padding (GAMES_GRID_FRAME(gridframe), 1, 1);
 	gtk_container_add (GTK_CONTAINER (gridframe), draw_area);
 	gtk_table_attach_defaults (GTK_TABLE (table), gridframe, 0, 1, 1, 10);
 
-	gtk_widget_set_events (draw_area, gtk_widget_get_events(draw_area) |GDK_BUTTON_PRESS_MASK);
+	gtk_widget_set_events (draw_area, gtk_widget_get_events(draw_area) |GDK_BUTTON_PRESS_MASK|GDK_KEY_PRESS_MASK);
+
+	GTK_WIDGET_SET_FLAGS (draw_area, GTK_CAN_FOCUS);
+	gtk_widget_grab_focus (draw_area);
 
 	update_score_state ();
 
