@@ -32,6 +32,7 @@
 #include "mahjongg.h"
 #include "drawing.h"
 #include "solubility.h"
+#include "maps.h"
 
 #define APPNAME "mahjongg"
 #define APPNAME_LONG "Mahjongg"
@@ -43,673 +44,6 @@
 /* The number of half-cycles to blink during a hint (less 1) */
 #define HINT_BLINK_NUM 5
 
-/* Sorted such that the bottom leftest are first, and layers decrease
- * Bottom left = high y, low x ! */
-
-tilepos easy_map [MAX_TILES] = {
- {13, 7,  4}, {12, 8,  3}, {14, 8,  3}, {12, 6,  3},
- {14, 6,  3}, {10, 10,  2}, {12, 10,  2}, {14, 10,  2},
- {16, 10,  2}, {10, 8,  2}, {12, 8,  2}, {14, 8,  2},
- {16, 8,  2}, {10, 6,  2}, {12, 6,  2}, {14, 6,  2},
- {16, 6,  2}, {10, 4,  2}, {12, 4,  2}, {14, 4,  2},
- {16, 4,  2}, {8, 12,  1}, {10, 12,  1}, {12, 12,  1},
- {14, 12,  1}, {16, 12,  1}, {18, 12,  1}, {8, 10,  1},
- {10, 10,  1}, {12, 10,  1}, {14, 10,  1}, {16, 10,  1}, 
- {18, 10,  1}, {8, 8,  1}, {10, 8,  1}, {12, 8,  1},
- {14, 8,  1}, {16, 8,  1}, {18, 8,  1}, {8, 6,  1},
- {10, 6,  1}, {12, 6,  1}, {14, 6,  1}, {16, 6,  1},
- {18, 6,  1}, {8, 4,  1}, {10, 4,  1}, {12, 4,  1},
- {14, 4,  1}, {16, 4,  1}, {18, 4,  1}, {8, 2,  1},
- {10, 2,  1}, {12, 2,  1}, {14, 2,  1}, {16, 2,  1},
- {18, 2,  1}, {2, 14,  0}, {4, 14,  0}, {6, 14,  0},
- {8, 14,  0}, {10, 14,  0}, {12, 14,  0}, {14, 14,  0}, 
- {16, 14,  0}, {18, 14,  0}, {20, 14,  0}, {22, 14,  0},
- {24, 14,  0}, {6, 12,  0}, {8, 12,  0}, {10, 12,  0},
- {12, 12,  0}, {14, 12,  0}, {16, 12,  0}, {18, 12,  0},
- {20, 12,  0}, {4, 10,  0}, {6, 10,  0}, {8, 10,  0},
- {10, 10,  0}, {12, 10,  0}, {14, 10,  0}, {16, 10,  0},
- {18, 10,  0}, {20, 10,  0}, {22, 10,  0}, {0, 7,  0},
- {2, 8,  0}, {4, 8,  0}, {6, 8,  0}, {8, 8,  0},
- {10, 8,  0}, {12, 8,  0}, {14, 8,  0}, {16, 8,  0},
- {18, 8,  0}, {20, 8,  0}, {22, 8,  0}, {24, 8,  0},
- {2, 6,  0}, {4, 6,  0}, {6, 6,  0}, {8, 6,  0},
- {10, 6,  0}, {12, 6,  0}, {14, 6,  0}, {16, 6,  0},
- {18, 6,  0}, {20, 6,  0}, {22, 6,  0}, {24, 6,  0},
- {4, 4,  0}, {6, 4,  0}, {8, 4,  0}, {10, 4,  0},
- {12, 4,  0}, {14, 4,  0}, {16, 4,  0}, {18, 4,  0},
- {20, 4,  0}, {22, 4,  0}, {6, 2,  0}, {8, 2,  0},
- {10, 2,  0}, {12, 2,  0}, {14, 2,  0}, {16, 2,  0},
- {18, 2,  0}, {20, 2,  0}, {2, 0,  0}, {4, 0,  0},
- {6, 0,  0}, {8, 0,  0}, {10, 0,  0}, {12, 0,  0},
- {14, 0,  0}, {16, 0,  0}, {18, 0,  0}, {20, 0,  0},
- {22, 0,  0}, {24, 0,  0}, {26, 7, 0}, {28, 7, 0} 
-};
-
-tilepos hard_map [MAX_TILES] = {
-	{ 10, 6,  6},
-	{ 9, 6,  5},
-	{ 11, 6,  5},
-	{ 8, 6,  4},
-	{ 10, 6,  4},
-	{ 12, 6,  4},
-	{ 5, 6,  3},
-	{ 7, 7,  3},
-	{ 7, 5,  3},
-	{ 9, 7,  3},
-	{ 9, 5,  3},
-	{ 11, 7,  3},
-	{ 11, 5,  3},
-	{ 13, 7,  3},
-	{ 13, 5,  3},
-	{ 15, 6,  3},
-	{ 5, 8,  2},
-	{ 7, 8,  2},
-	{ 9, 8,  2},
-	{ 11, 8,  2},
-	{ 13, 8,  2},
-	{ 15, 8,  2},
-	{ 4, 6,  2},
-	{ 6, 6,  2},
-	{ 8, 6,  2},
-	{ 10, 6,  2},
-	{ 12, 6,  2},
-	{ 14, 6,  2},
-	{ 16, 6,  2},
-	{ 5, 4,  2},
-	{ 7, 4,  2},
-	{ 9, 4,  2},
-	{ 11, 4,  2},
-	{ 13, 4,  2},
-	{ 15, 4,  2},
-	{ 7, 12,  1},
-	{ 9, 11,  1},
-	{ 11, 11,  1},
-	{ 13, 12,  1},
-	{ 2, 10,  1},
-	{ 4, 10,  1},
-	{ 6, 10,  1},
-	{ 8, 9,  1},
-	{ 10, 9,  1},
-	{ 12, 9,  1},
-	{ 14, 10,  1},
-	{ 16, 10,  1},
-	{ 18, 10,  1},
-	{ 3, 8,  1},
-	{ 3, 6,  1},
-	{ 5, 8,  1},
-	{ 5, 6,  1},
-	{ 7, 7,  1},
-	{ 9, 7,  1},
-	{ 11, 7,  1},
-	{ 13, 7,  1},
-	{ 15, 8,  1},
-	{ 17, 8,  1},
-	{ 3, 4,  1},
-	{ 5, 4,  1},
-	{ 7, 5,  1},
-	{ 9, 5,  1},
-	{ 11, 5,  1},
-	{ 13, 5,  1},
-	{ 15, 6,  1},
-	{ 17, 6,  1},
-	{ 2, 2,  1},
-	{ 4, 2,  1},
-	{ 6, 2,  1},
-	{ 8, 3,  1},
-	{ 10, 3,  1},
-	{ 12, 3,  1},
-	{ 15, 4,  1},
-	{ 17, 4,  1},
-	{ 7, 0,  1},
-	{ 9, 1,  1},
-	{ 11, 1,  1},
-	{ 14, 2,  1},
-	{ 16, 2,  1},
-	{ 18, 2,  1},
-	{ 13, 0,  1},
-	{ 6, 12,  0},
-	{ 8, 12,  0},
-	{ 10, 12,  0},
-	{ 12, 12,  0},
-	{ 14, 12,  0},
-	{ 1, 11,  0},
-	{ 3, 11,  0},
-	{ 1, 9,  0},
-	{ 0, 6,  0},
-	{ 3, 9,  0},
-	{ 5, 10,  0},
-	{ 7, 10,  0},
-	{ 9, 10,  0},
-	{ 11, 10,  0},
-	{ 13, 10,  0},
-	{ 15, 10,  0},
-	{ 17, 11,  0},
-	{ 19, 11,  0},
-	{ 2, 7,  0},
-	{ 4, 7,  0},
-	{ 6, 8,  0},
-	{ 8, 8,  0},
-	{ 2, 5,  0},
-	{ 4, 5,  0},
-	{ 6, 6,  0},
-	{ 8, 6,  0},
-	{ 10, 8,  0},
-	{ 10, 6,  0},
-	{ 12, 8,  0},
-	{ 12, 6,  0},
-	{ 14, 8,  0},
-	{ 14, 6,  0},
-	{ 17, 9,  0},
-	{ 16, 7,  0},
-	{ 19, 9,  0},
-	{ 18, 7,  0},
-	{ 1, 3,  0},
-	{ 3, 3,  0},
-	{ 6, 4,  0},
-	{ 8, 4,  0},
-	{ 10, 4,  0},
-	{ 12, 4,  0},
-	{ 14, 4,  0},
-	{ 16, 5,  0},
-	{ 18, 5,  0},
-	{ 20, 6,  0},
-	{ 1, 1,  0},
-	{ 3, 1,  0},
-	{ 5, 2,  0},
-	{ 7, 2,  0},
-	{ 9, 2,  0},
-	{ 11, 2,  0},
-	{ 13, 2,  0},
-	{ 15, 2,  0},
-	{ 17, 3,  0},
-	{ 19, 3,  0},
-	{ 17, 1,  0},
-	{ 19, 1,  0},
-	{ 6, 0,  0},
-	{ 8, 0,  0},
-	{ 10, 0,  0},
-	{ 12, 0,  0},
-	{ 14, 0,  0}
-};
-
-tilepos pyramid_map [MAX_TILES] = {
- {13, 14, 6}, {13, 8,  6},  {13, 2, 6}, {12, 14,  5},
- {14, 14,  5}, {12, 8,  5}, {14, 8,  5}, {12, 2,  5},
- {14, 2,  5}, {10, 14,  4}, {12, 14,  4}, {14, 14,  4},
- {16, 14,  4}, {2, 8,  4}, {11, 8,  4}, {13, 8,  4},
- {15, 8,  4}, {24, 8,  4}, {10, 2,  4}, {12, 2,  4},
- {14, 2,  4}, {16, 2,  4}, {8, 14,  3}, {10, 14,  3},
- {12, 14,  3}, {14, 14,  3}, {16, 14,  3}, {18, 14,  3},
- {2, 9,  3}, {24, 9, 3}, {10, 8,  3}, {12, 8,  3},
- {14, 8,  3}, {16, 8, 3}, {2, 7,  3}, {24, 7,  3},
- {8, 2,  3}, {10, 2,  3}, {12, 2,  3}, {14, 2,  3},
- {16, 2,  3}, {18, 2,  3}, {6, 14,  2}, {8, 14,  2},
- {10, 14,  2}, {12, 14,  2}, {14, 14,  2}, {16, 14,  2},
- {18, 14,  2}, {20, 14,  2}, {2, 10,  2}, {24, 10,  2},
- {2, 8,  2}, {9, 8,  2}, {11, 8,  2}, {13, 8,  2},
- {15, 8,  2}, {17, 8,  2}, {24, 8,  2}, {2, 6,  2},
- {24, 6,  2}, {6, 2,  2}, {8, 2,  2}, {10, 2,  2},
- {12, 2,  2}, {14, 2,  2}, {16, 2,  2}, {18, 2,  2},
- {20, 2,  2}, {4, 14,  1}, {6, 14,  1}, {8, 14,  1},
- {10, 14,  1}, {12, 14,  1}, {14, 14,  1}, {16, 14,  1},
- {18, 14,  1}, {20, 14,  1}, {22, 14,  1}, {2, 11,  1},
- {24, 11,  1}, {2, 9,  1}, {24, 9,  1}, {8, 8,  1},
- {10, 8,  1}, {12, 8,  1}, {14, 8,  1}, {16, 8,  1},
- {18, 8,  1}, {2, 7,  1}, {24, 7,  1}, {2, 5,  1},
- {24, 5,  1}, {4, 2,  1}, {6, 2,  1}, {8, 2,  1},
- {10, 2,  1}, {12, 2,  1}, {14, 2,  1}, {16, 2,  1},
- {18, 2,  1}, {20, 2,  1}, {22, 2,  1}, {2, 14,  0},
- {4, 14,  0}, {6, 14,  0}, {8, 14,  0}, {10, 14,  0},
- {12, 14,  0}, {14, 14,  0}, {16, 14,  0}, {18, 14,  0},
- {20, 14,  0}, {22, 14,  0}, {24, 14,  0}, {2, 12,  0},
- {24, 12,  0}, {2, 10,  0}, {24, 10,  0}, {2, 8, 0},
- {7, 8,  0}, {9, 8,  0}, {11, 8,  0}, {13, 8,  0},
- {15, 8,  0}, {17, 8,  0}, {19, 8,  0}, {24, 8, 0},
- {2, 6,  0}, {24, 6,  0}, {2, 4,  0}, {24, 4,  0},
- {2, 2,  0}, {4, 2,  0}, {6, 2,  0}, {8, 2,  0},
- {10, 2,  0}, {12, 2,  0}, {14, 2,  0}, {16, 2,  0},
- {18, 2,  0}, {20, 2,  0}, {22, 2,  0}, {24, 2,  0}
-};
-
-tilepos cross_map [MAX_TILES] = {
- {13, 8, 5},
- {13, 12,  4}, {13, 10,  4}, {9, 8,  4}, {11, 8,  4},
- {13, 8,  4}, {15, 8,  4}, {17, 8,  4}, {13, 6,  4},
- {13, 4,  4}, {5, 14,  3}, {21, 14,  3}, {13, 13,  3},
- {13, 11,  3}, {6, 8,  3},
- {8, 8, 3}, {10, 8,  3}, {12, 9,  3}, {12, 7,  3}, {14, 9,  3}, {14, 7,  3},  {16, 8,  3}, {18, 8,  3}, {20, 8, 3},
-   {13, 5,  3},
- {13, 3,  3}, {5, 2,  3}, {21, 2,  3}, {5, 14,  2},
- {7, 14,  2}, {13, 14,  2}, {19, 14,  2}, {21, 14,  2},
- {5, 12,  2}, {13, 12,  2}, {21, 12,  2}, {13, 10, 2},
- {5, 8, 2}, {7, 8,  2}, {9, 8,  2}, {11, 8,  2},
- {13, 8,  2}, {15, 8,  2}, {17, 8,  2}, {19, 8,  2},
- {21, 8, 2}, {13, 6, 2}, {5, 4,  2}, {13, 4,  2},
- {21, 4,  2}, {5, 2,  2}, {7, 2,  2}, {13, 2,  2},
- {19, 2,  2}, {21, 2,  2}, {13, 15,  1}, {5, 14,  1},
- {7, 14,  1}, {9, 14,  1}, {17, 14,  1}, {19, 14,  1},
- {21, 14,  1}, {13, 13,  1}, {5, 12,  1}, {7, 12, 1},
- {19, 12, 1}, {21, 12,  1}, {13, 11,  1}, {5, 10,  1},
- {21, 10,  1}, {4, 8,  1},
- {6, 8,  1}, {8, 8,  1}, {10, 8,  1}, {12, 9,  1}, {12, 7,  1},
- {14, 9,  1}, {14, 7,  1},  {16, 8,  1}, {18, 8,  1}, {20, 8,  1},
- {22, 8,  1}, {5, 6,  1}, {21, 6,  1}, {13, 5,  1},
- {5, 4,  1}, {7, 4, 1}, {19, 4, 1}, {21, 4,  1},
- {13, 3,  1}, {5, 2,  1}, {7, 2,  1}, {9, 2,  1},
- {17, 2,  1},{19, 2,  1}, {21, 2,  1}, {13, 1,  1},
- {13, 16,  0}, {5, 14,  0}, {7, 14,  0}, {9, 14,  0},
- {11, 14,  0}, {13, 14,  0}, {15, 14,  0}, {17, 14,  0},
- {19, 14,  0}, {21, 14,  0}, {5, 12,  0}, {7, 12, 0},
- {13, 12,  0}, {19, 12, 0}, {21, 12,  0}, {5, 10,  0},
- {13, 10,  0}, {21, 10,  0}, {3, 8, 0},{5, 8, 0},
- {7, 8,  0}, {9, 8,  0}, {11, 8,  0}, {13, 8,  0},
- {15, 8,  0}, {17, 8,  0}, {19, 8,  0}, {21, 8, 0},
- {23, 8,  0}, {5, 6,  0}, {13, 6,  0}, {21, 6,  0},
- {5, 4,  0}, {7, 4, 0}, {13, 4,  0}, {19, 4, 0},
- {21, 4,  0}, {5, 2,  0}, {7, 2,  0}, {9, 2,  0},
- {11, 2,  0}, {13, 2,  0}, {15, 2,  0}, {17, 2,  0},
- {19, 2,  0}, {21, 2,  0}, {13, 0, 0}
-};
-
-
-tilepos tictactoe_map [MAX_TILES] = {
- {6, 12,  4},{8, 12,  4}, {10, 12,  4}, {12, 12,  4}, {14, 12,  4}, {16, 12, 4},{18, 12,  4},
- {6, 10,  4}, {18, 10,  4},
- {6, 8,  4}, {18, 8,  4},
- {6, 6,  4}, {18, 6,  4},
- {6, 4,  4}, {8, 4,  4}, {10, 4,  4}, {12, 4,  4}, {14, 4,  4}, {16, 4, 4},{18, 4,  4},
-
- {6, 12,  3}, {8, 12,  3}, {10, 12,  3}, {12, 12,  3}, {14, 12,  3}, {16, 12, 3}, {18, 12,  3},
- {6, 10,  3}, {18, 10,  3},
- {6, 8,  3}, {18, 8,  3},
- {6, 6,  3}, {18, 6,  3},
- {6, 4,  3}, {8, 4,  3}, {10, 4,  3}, {12, 4,  3}, {14, 4,  3}, {16, 4,  3}, {18, 4,  3},
-
- {6, 14,  2}, {18, 14,  2},
- {4, 12,  2}, {6, 12,  2}, {8, 12,  2}, {10, 12,  2}, {12, 12,  2}, {14, 12, 2}, {16, 12,  2}, {18, 12,  2}, {20, 12,  2},
- {6, 10,  2}, {18, 10,  2},
- {6, 8,  2}, {18, 8,  2},
- {6, 6,  2}, {18, 6,  2},
- {4, 4,  2}, {6, 4,  2}, {8, 4,  2}, {10, 4,  2}, {12, 4,  2}, {14, 4,  2}, {16, 4,  2}, {18, 4,  2}, {20, 4,  2},
- {6, 2,  2}, {18, 2,  2},
-
- {6, 16,  1}, {18, 16,  1},
- {6, 14,  1}, {18, 14,  1},
- {2, 12,  1}, {4, 12,  1}, {6, 12,  1}, {8, 12,  1}, {10, 12,  1}, {12, 12, 1}, {14, 12,  1}, {16, 12,  1}, {18, 12,  1}, {20, 12,  1}, {22, 12,  1},
- {6, 10,  1}, {18, 10,  1},
- {6, 8,  1}, {18, 8,  1},
- {6, 6,  1}, {18, 6,  1},
- {2, 4,  1}, {4, 4,  1}, {6, 4,  1}, {8, 4,  1}, {10, 4,  1}, {12, 4,  1}, {14, 4,  1}, {16, 4,  1}, {18, 4,  1}, {20, 4,  1}, {22, 4,  1},
- {6, 2,  1}, {18, 2,  1},
- {6, 0,  1}, {18, 0,  1},
-
- {6, 16,  0}, {18, 16,  0},
- {6, 14,  0}, {18, 14,  0},
- {0, 12, 0}, {2, 12,  0}, {4, 12,  0}, {6, 12,  0}, {8, 12,  0}, {10, 12,  0}, {12, 12,  0}, {14, 12,  0}, {16, 12,  0},{18, 12,  0}, {20, 12,  0},{22, 12, 0}, {24, 12, 0},
- {6, 10,  0}, {18, 10,  0},
- {6, 8,  0}, {18, 8,  0},
- {6, 6,  0}, {18, 6,  0},
- {0, 4, 0}, {2, 4,  0}, {4, 4,  0}, {6, 4,  0}, {8, 4,  0}, {10, 4,  0}, {12, 4,  0}, {14, 4,  0}, {16, 4,  0},{18, 4,  0}, {20, 4,  0},{22, 4,  0}, {24, 4, 0},
- {6, 2,  0}, {18, 2,  0},
- {6, 0,  0}, {18, 0,  0},
-};
-
-tilepos cloud_map [MAX_TILES] = {
- {12, 11,  2},
-
- {0, 6, 2}, {4, 6, 2}, {8, 6, 2}, {12, 6, 2}, {16, 6, 2}, {20, 6, 2}, {24, 6, 2},
- {0, 4, 2}, {4, 4, 2}, {8, 4, 2}, {12, 4, 2}, {16, 4, 2}, {20, 4, 2}, {24, 4, 2},
- {0, 2, 2}, {4, 2, 2}, {8, 2, 2}, {12, 2, 2}, {16, 2, 2}, {20, 2, 2}, {24, 2, 2},
- {0, 0, 2}, {4, 0, 2}, {8, 0, 2}, {12, 0, 2}, {16, 0, 2}, {20, 0, 2}, {24, 0, 2},
-
- {6, 11, 1}, {8, 11, 1}, {10, 11, 1}, {12, 11, 1}, {14, 11, 1},
- {16, 11, 1}, {18, 11, 1}, {20, 11, 1},
-
- {0, 6, 1}, {4, 6, 1}, {8, 6, 1}, {12, 6, 1}, {16, 6, 1}, {20, 6, 1}, {24, 6, 1},
- {0, 4, 1}, {4, 4, 1}, {8, 4, 1}, {12, 4, 1}, {16, 4, 1}, {20, 4, 1}, {24, 4, 1},
- {0, 2, 1}, {4, 2, 1}, {8, 2, 1}, {12, 2, 1}, {16, 2, 1}, {20, 2, 1}, {24, 2, 1},
- {0, 0, 1}, {4, 0, 1}, {8, 0, 1}, {12, 0, 1}, {16, 0, 1}, {20, 0, 1}, {24, 0, 1},
-
- {5, 11, 0}, {7, 11, 0}, {9, 11, 0}, {11, 11, 0}, {13, 11, 0},
- {15, 11, 0}, {17, 11, 0}, {19, 11, 0}, {21, 11, 0},
-
- {0, 8, 0}, {2, 8, 0}, {4, 8, 0}, {6, 8, 0}, {8, 8, 0}, {10, 8, 0}, {12, 8, 0},
- {14, 8, 0}, {16, 8, 0}, {18, 8, 0}, {20, 8, 0}, {22, 8, 0}, {24, 8, 0}, {26, 8, 0},
-
- {0, 6, 0}, {2, 6, 0}, {4, 6, 0}, {6, 6, 0}, {8, 6, 0}, {10, 6, 0}, {12, 6, 0},
- {14, 6, 0}, {16, 6, 0}, {18, 6, 0}, {20, 6, 0}, {22, 6, 0}, {24, 6, 0}, {26, 6, 0},
-
- {0, 4, 0}, {2, 4, 0}, {4, 4, 0}, {6, 4, 0}, {8, 4, 0}, {10, 4, 0}, {12, 4, 0},
- {14, 4, 0}, {16, 4, 0}, {18, 4, 0}, {20, 4, 0}, {22, 4, 0}, {24, 4, 0}, {26, 4, 0},
-
- {0, 2, 0}, {2, 2, 0}, {4, 2, 0}, {6, 2, 0}, {8, 2, 0}, {10, 2, 0}, {12, 2, 0},
- {14, 2, 0}, {16, 2, 0}, {18, 2, 0}, {20, 2, 0}, {22, 2, 0}, {24, 2, 0}, {26, 2, 0},
-
- {0, 0, 0}, {2, 0, 0}, {4, 0, 0}, {6, 0, 0}, {8, 0, 0}, {10, 0, 0}, {12, 0, 0},
- {14, 0, 0}, {16, 0, 0}, {18, 0, 0}, {20, 0, 0}, {22, 0, 0}, {24, 0, 0}, {26, 0, 0},
-
-};
-
-tilepos reddragon_map [MAX_TILES] = {
-{10,8,2},
-{12,8,2},
-{14,8,2},
-{16,8,2},
-{22,8,2},
-{10,6,2},
-{12,6,2},
-{14,6,2},
-{16,6,2},
-{10,4,2},
-{12,4,2},
-{14,4,2},
-{16,4,2},
-{10,2,2},
-{12,2,2},
-{14,2,2},
-{16,2,2},
-{5,9,1},
-{7,9,1},
-{9,9,1},
-{11,9,1},
-{13,9,1},
-{15,9,1},
-{17,9,1},
-{19,9,1},
-{21,9,1},
-{7,7,1},
-{9,7,1},
-{11,7,1},
-{13,7,1},
-{15,7,1},
-{17,7,1},
-{19,7,1},
-{21,7,1},
-{23,7,1},
-{7,5,1},
-{9,5,1},
-{11,5,1},
-{13,5,1},
-{15,5,1},
-{17,5,1},
-{19,5,1},
-{21,5,1},
-{23,5,1},
-{5,3,1},
-{7,3,1},
-{9,3,1},
-{11,3,1},
-{13,3,1},
-{15,3,1},
-{17,3,1},
-{19,3,1},
-{21,3,1},
-{23,3,1},
-{7,1,1},
-{9,1,1},
-{11,1,1},
-{13,1,1},
-{15,1,1},
-{17,1,1},
-{19,1,1},
-{21,1,1},
-{4,13,0},
-{8,13,0},
-{12,13,0},
-{16,13,0},
-{20,13,0},
-{24,13,0},
-{0,12,0},
-{28,12,0},
-{4,10,0},
-{6,10,0},
-{8,10,0},
-{10,10,0},
-{12,10,0},
-{14,10,0},
-{16,10,0},
-{18,10,0},
-{20,10,0},
-{22,10,0},
-{24,10,0},
-{0,9,0},
-{28,9,0},
-{4,8,0},
-{6,8,0},
-{8,8,0},
-{10,8,0},
-{12,8,0},
-{14,8,0},
-{16,8,0},
-{18,8,0},
-{20,8,0},
-{22,8,0},
-{24,8,0},
-{0,6,0},
-{4,6,0},
-{6,6,0},
-{8,6,0},
-{10,6,0},
-{12,6,0},
-{14,6,0},
-{16,6,0},
-{18,6,0},
-{20,6,0},
-{22,6,0},
-{24,6,0},
-{28,6,0},
-{4,4,0},
-{6,4,0},
-{8,4,0},
-{10,4,0},
-{12,4,0},
-{14,4,0},
-{16,4,0},
-{18,4,0},
-{20,4,0},
-{22,4,0},
-{24,4,0},
-{0,3,0},
-{28,3,0},
-{4,2,0},
-{6,2,0},
-{8,2,0},
-{10,2,0},
-{12,2,0},
-{14,2,0},
-{16,2,0},
-{18,2,0},
-{20,2,0},
-{22,2,0},
-{24,2,0},
-{0,0,0},
-{4,0,0},
-{6,0,0},
-{8,0,0},
-{10,0,0},
-{12,0,0},
-{14,0,0},
-{16,0,0},
-{18,0,0},
-{20,0,0},
-{22,0,0},
-{24,0,0},
-{28,0,0}
-};
-tilepos fourbridges_map [MAX_TILES] = {
-       {7,13,3},
-       {17,13,3},
-       {7,3,3},
-       {17,3,3},
-       {6,14,2},
-       {8,14,2},
-       {16,14,2},
-       {18,14,2},
-       {6,12,2},
-       {8,12,2},
-       {16,12,2},
-       {18,12,2},
-       {6,4,2},
-       {8,4,2},
-       {16,4,2},
-       {18,4,2},
-       {6,2,2},
-       {8,2,2},
-       {16,2,2},
-       {18,2,2},
-       {5,15,1},
-       {7,15,1},
-       {9,15,1},
-       {15,15,1},
-       {17,15,1},
-       {19,15,1},
-       {5,13,1},
-       {7,13,1},
-       {9,13,1},
-       {15,13,1},
-       {17,13,1},
-       {19,13,1},
-       {5,11,1},
-       {7,11,1},
-       {9,11,1},
-       {15,11,1},
-       {17,11,1},
-       {19,11,1},
-       {5,5,1},
-       {7,5,1},
-       {9,5,1},
-       {15,5,1},
-       {17,5,1},
-       {19,5,1},
-       {5,3,1},
-       {7,3,1},
-       {9,3,1},
-       {15,3,1},
-       {17,3,1},
-       {19,3,1},
-       {5,1,1},
-       {7,1,1},
-       {9,1,1},
-       {15,1,1},
-       {17,1,1},
-       {19,1,1},
-       {2,16,0},
-       {4,16,0},
-       {6,16,0},
-       {8,16,0},
-       {10,16,0},
-       {12,16,0},
-       {14,16,0},
-       {16,16,0},
-       {18,16,0},
-       {20,16,0},
-       {22,16,0},
-       {4,14,0},
-       {6,14,0},
-       {8,14,0},
-       {10,14,0},
-       {14,14,0},
-       {16,14,0},
-       {18,14,0},
-       {20,14,0},
-       {4,12,0},
-       {6,12,0},
-       {8,12,0},
-       {10,12,0},
-       {12,12,0},
-       {14,12,0},
-       {16,12,0},
-       {18,12,0},
-       {20,12,0},
-       {0,10,0},
-       {2,10,0},
-       {4,10,0},
-       {6,10,0},
-       {8,10,0},
-       {10,10,0},
-       {12,10,0},
-       {14,10,0},
-       {16,10,0},
-       {18,10,0},
-       {20,10,0},
-       {22,10,0},
-       {24,10,0},
-       {3,8,0},
-       {5,8,0},
-       {7,8,0},
-       {17,8,0},
-       {19,8,0},
-       {21,8,0},
-       {0,6,0},
-       {2,6,0},
-       {4,6,0},
-       {6,6,0},
-       {8,6,0},
-       {10,6,0},
-       {12,6,0},
-       {14,6,0},
-       {16,6,0},
-       {18,6,0},
-       {20,6,0},
-       {22,6,0},
-       {24,6,0},
-       {4,4,0},
-       {6,4,0},
-       {8,4,0},
-       {10,4,0},
-       {12,4,0},
-       {14,4,0},
-       {16,4,0},
-       {18,4,0},
-       {20,4,0},
-       {4,2,0},
-       {6,2,0},
-       {8,2,0},
-       {10,2,0},
-       {14,2,0},
-       {16,2,0},
-       {18,2,0},
-       {20,2,0},
-       {2,0,0},
-       {4,0,0},
-       {6,0,0},
-       {8,0,0},
-       {10,0,0},
-       {12,0,0},
-       {14,0,0},
-       {16,0,0},
-       {18,0,0},
-       {20,0,0},
-       {22,0,0}
-};
-
-
-
-tilepos ziggurat_map [MAX_TILES] = {
- {10, 8, 5}, {12, 8, 5}, {14, 8, 5}, {16, 8, 5}, {18, 8, 5},
- {10, 6, 5}, {12, 6, 5}, {14, 6, 5}, {16, 6, 5}, {18, 6, 5},
- {8, 8, 4}, {10, 8, 4}, {12, 8, 4}, {14, 8, 4}, {16, 8, 4}, {18, 8, 4}, {20, 8, 4},
- {8, 6, 4}, {10, 6, 4}, {12, 6, 4}, {14, 6, 4}, {16, 6, 4}, {18, 6, 4}, {20, 6, 4},
- {6, 8, 3}, {8, 8, 3}, {10, 8, 3}, {12, 8, 3}, {14, 8, 3}, {16, 8, 3}, {18, 8, 3}, {20, 8, 3}, {22, 8, 3},
- {6, 6, 3}, {8, 6, 3}, {10, 6, 3}, {12, 6, 3}, {14, 6, 3}, {16, 6, 3}, {18, 6, 3}, {20, 6, 3}, {22, 6, 3},
- {6, 13,  2},  {22, 13,  2}, {14, 12,  2},
- {4, 8,  2}, {6, 8, 2}, {8, 8, 2}, {20, 8, 2}, {22, 8, 2}, {24, 8, 2},
- {4, 6,  2}, {6, 6, 2}, {8, 6, 2}, {20, 6, 2}, {22, 6, 2}, {24, 6, 2},
- {14, 2,  2}, {6, 1,  2}, {22, 1,  2},
- {6, 13,  1},  {22, 13,  1}, {13, 12,  1}, {15, 12,  1},
- {2, 8, 1}, {4, 8,  1}, {6, 8, 1}, {22, 8,  1}, {24, 8,  1}, {26, 8,  1},
- {2, 6, 1}, {4, 6,  1}, {6, 6, 1}, {22, 6,  1}, {24, 6,  1}, {26, 6,  1},
- {13, 2,  1}, {15, 2,  1}, {6, 1,  1}, {22, 1,  1},
- {0,14,0}, {5, 14,  0}, {7, 14,  0}, {9, 14,  0}, {11, 14,  0}, {13, 14, 0}, {15, 14,  0}, {17, 14, 0}, {19, 14,  0}, {21, 14, 0}, {23, 14,  0}, {28,14,0},
- {0,12,0}, {5, 12,  0},{13,12,0},{15,12,0},{23, 12,  0},{28,12,0},
- {0,10,0}, {5, 10,  0}, {13, 10, 0}, {15, 10,  0}, {23, 10,  0},{28,10,0},
- {0, 8,  0}, {2, 8, 0}, {4, 8,  0}, {6, 8, 0}, {22, 8,  0}, {24, 8,  0}, {26, 8,  0}, {28, 8,  0},
- {0, 6,  0}, {2, 6, 0}, {4, 6,  0}, {6, 6, 0}, {22, 6,  0}, {24, 6,  0}, {26, 6,  0}, {28, 6,  0},
- {0,4,0}, {5, 4,  0}, {13, 4,  0}, {15, 4,  0}, {23, 4,  0},{28,4,0},
- {0,2,0},{5, 2,  0}, {13,2,0},{15,2,0},{23, 2,  0},{28,2,0},
- {0,0,0},{5, 0,  0}, {7, 0,  0}, {9, 0,  0}, {11, 0,  0}, {13, 0, 0}, {15, 0,  0}, {17, 0,  0}, {19, 0,  0}, {21, 0, 0}, {23, 0,  0},{28,0,0},
-};
-
 tilepos *pos = 0;
 
 tile tiles[MAX_TILES];
@@ -729,23 +63,8 @@ static gchar *score_current_mapset = NULL;
 
 static gchar *selected_tileset = NULL;
 
-struct _maps
-{
-  gchar *name ;
-  gchar *score_name;
-  tilepos *map ;
-} maps[] = {
-	{ N_("Easy"), "easy", easy_map },
-	{ N_("Difficult"), "difficult", hard_map },
-        { N_("Confounding Cross"), "confounding", cross_map },
-        { N_("Pyramid's Walls"), "pyramid", pyramid_map },
-        { N_("Tic-Tac-Toe"), "tictactoe", tictactoe_map },
-        { N_("Cloud"), "cloud", cloud_map },
-        { N_("Red Dragon"), "dragon", reddragon_map },
-        { N_("Four Bridges"), "bridges", fourbridges_map },
-	{ N_("The Ziggurat"), "ziggurat", ziggurat_map },
-};
-
+gboolean undo_state = FALSE;
+gboolean redo_state = FALSE;
 
 gint hint_tiles[2];
 guint timer;
@@ -897,6 +216,9 @@ static void set_menus_sensitive (void)
 /* Undo and redo sensitivity functionality. */
 static void set_undoredo_sensitive (gboolean undo, gboolean redo)
 {
+	undo_state = undo;
+	redo_state = redo;
+
 	gtk_widget_set_sensitive(gamemenu[4].widget, undo);
 	gtk_widget_set_sensitive(toolbar_uiinfo[4].widget, undo);
 	gtk_widget_set_sensitive(gamemenu[5].widget, redo);
@@ -1059,6 +381,7 @@ mapset_changed_cb (GConfClient *client,
 	mapset_tmp = gconf_client_get_string (conf_client,
 			"/apps/mahjongg/mapset",
 			NULL);
+
 	/* We check whether the name is valid later. */
 	if (mapset_tmp != NULL) {
 		g_free (mapset);
@@ -1090,9 +413,9 @@ mapset_changed_cb (GConfClient *client,
 static void
 set_map_selection (GtkWidget *widget, void *data)
 {
-	struct _maps *map;
+	map *map;
 
-	map = &(maps[gtk_combo_box_get_active (GTK_COMBO_BOX (widget))]);
+	map = maps + gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
 	
 	g_free (mapset);
 	mapset = g_strdup (map->name);
@@ -1313,7 +636,7 @@ fill_map_menu (GtkWidget *menu)
 {
 	gint lp, itemno=0 ;
 
-	for (lp=0;lp<G_N_ELEMENTS(maps);lp++) {
+	for (lp=0;lp<nmaps;lp++) {
 		gchar *str = g_strdup(_(maps[lp].name)) ;
 
 		gtk_combo_box_append_text (GTK_COMBO_BOX (menu), str) ;
@@ -1717,12 +1040,24 @@ pause_callback (void)
         if (paused) {
 		gtk_toggle_button_set_active (
                   GTK_TOGGLE_BUTTON (toolbar_uiinfo[2].widget), TRUE);
+		gtk_widget_set_sensitive(gamemenu[4].widget, FALSE);
+		gtk_widget_set_sensitive(toolbar_uiinfo[4].widget, FALSE);
+		gtk_widget_set_sensitive(gamemenu[5].widget, FALSE);
+		gtk_widget_set_sensitive(toolbar_uiinfo[5].widget, FALSE);
+		gtk_widget_set_sensitive(gamemenu[6].widget, FALSE);
+		gtk_widget_set_sensitive(toolbar_uiinfo[6].widget, FALSE);
                 games_clock_stop (GAMES_CLOCK (chrono));
                 message(_("Game paused"));
         }
         else {
 		gtk_toggle_button_set_active (
                   GTK_TOGGLE_BUTTON (toolbar_uiinfo[2].widget), FALSE);
+		gtk_widget_set_sensitive(gamemenu[4].widget, undo_state);
+		gtk_widget_set_sensitive(toolbar_uiinfo[4].widget, undo_state);
+		gtk_widget_set_sensitive(gamemenu[5].widget, redo_state);
+		gtk_widget_set_sensitive(toolbar_uiinfo[5].widget, redo_state);
+		gtk_widget_set_sensitive(gamemenu[6].widget, TRUE); 
+		gtk_widget_set_sensitive(toolbar_uiinfo[6].widget, TRUE);
                 games_clock_start (GAMES_CLOCK(chrono));
                 message ("");
         }
@@ -1731,11 +1066,8 @@ pause_callback (void)
 
 void ensure_pause_off (void)
 {
-        if (paused) {
-		draw_all_tiles ();
-                message("");
-        }
-        paused = FALSE;
+        if (paused)
+		pause_callback ();
 }
 
 void
@@ -1915,7 +1247,7 @@ load_map (void)
 	new_map = FALSE;
 		
 	found = FALSE;
-	for (lp=0;lp<G_N_ELEMENTS(maps);lp++)
+	for (lp=0;lp<nmaps;lp++)
 		if (g_ascii_strcasecmp (maps[lp].name, name) == 0) {
 			found = TRUE;
 			break;
@@ -2001,7 +1333,7 @@ static void set_score_file (gchar * mapset)
 	   map (and it isn't suitable for generating the scorefile
 	   name. It's also a bit close to code freeze to introduce
 	   gratuitous changes so this is it. */
-	for (i=0; i<G_N_ELEMENTS(maps); i++) {
+	for (i=0; i<nmaps; i++) {
 		if (g_utf8_collate (mapset, maps[i].name) == 0)
 			score_current_mapset = maps[i].score_name;
 	}
@@ -2092,6 +1424,8 @@ main (int argc, char *argv [])
 
 	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-mahjongg.png");
 
+	load_maps ();
+
 	conf_client = gconf_client_get_default ();
 	if (!games_gconf_sanity_check_string (conf_client, "/apps/mahjongg/tileset")) {
 		return 1;
